;;;;
;;;;     axisGLIM.lsp  Robert A. Stine, 1994
;;;;
;;;;             Add in module for augmenting Axis with interface
;;;;             to the glim.lsp code of Tierney.
#||
    17 Apr 94 ... Created to extend Axis.
     4 Jul 97 ... Patched for array mult
||#

(require "glim")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                         ;;
;;         Add commands to the statistics menu.            ;;
;;                                                         ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun ADD-GLIM-ITEMS ()
  (send *statistics-menu* :append-items
        (send stat-menu-item-proto :new
              "Logistic Regr" glim-cmd-proto)
        ))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                         ;;
;;          Generalized Linear Model Command               ;;
;;                                             17 Apr 94   ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defproto GLIM-CMD-PROTO
  '(joinToggle)
  '(classIDCode)       ;; required for every "type" of command
  commando-proto)


(defmeth glim-cmd-proto :ISNEW ()
  (call-next-method
        "Generalized Linear Model"
        'binomialreg-model
        (list "Response   ->"
              (list "Covariates ->")) 
   ))

(defmeth glim-cmd-proto :COMMAND ()
  (let* ((txt  (send self :text))   
         (form (send self :form))
         ( x   (select form 1))    )
    (cons
     (slot-value 'cmdForm)
     `( (list ,@x)
        ,(select form 0)
        1
        :response-name ,(first txt)
        :predictor-names (list ,@(second txt))
        :case-labels ^cases
        :included ^filter)
     )))


(defmeth glim-cmd-proto :DOIT ()
  (call-next-method)
  (setf *regression* (slot-value 'target))
  (send *regression* :display-error-rate)
  )

#|
  (def gc (send glim-cmd-proto :new))
|#


(defmeth glim-cmd-proto :BUTTON-ITEMS ()
  (let ((print-but (send button-item-proto :new "Print"
                         :action #'(lambda ()
                                     (send
                                      (send self :slot-value 'target)
                                      :display))))
        (resid-but (send button-item-proto :new "Plot Resids"
                         :action #'(lambda ()
                                     (send
                                      (send self :slot-value 'target)
                                      :plot-residuals))))
        (save-but  (send button-item-proto :new "Save Data"
                         :action #'(lambda ()
                                     (saver 
                                      (send self :slot-value 'target)
                                      (send self :dataset)))))
        (boot-but (send commando-button-proto :new "Bootstrap"
                        :action
                        #'(lambda (x)
                            (let ((resp (send self :boot-dialog)))
                              (if resp (apply #'send x :bootstrap resp))))))
        )
    (list (list print-but resid-but save-but boot-but))
    ))
        


(defmeth glim-cmd-proto :BOOT-DIALOG ()
  (let* ((prompt (send text-item-proto :new "Enter bootstrap B:"))
         (nReps  (send edit-text-item-proto :new "50" :text-length 5))
         (method (send choice-item-proto :new 
                       (list "Random X" "Fixed X") :value 0))
         (ok     (send modal-button-proto :new "OK"
                       :action #'(lambda ()
                                   (list (read (make-string-input-stream
                                                (send nReps :text)) nil)
                                         (select (list 'random 'fixed)
                                                 (send method :value)))) ))
         (cancel (send modal-button-proto :new "Cancel"
                       :action #'(lambda () nil)))
         (dialog  (send modal-dialog-proto :new 
                        (list (list prompt nReps)
                              (list method)
                              (list ok cancel))
                        :title "Bootstrap Settings"
                        :default-button ok))    )
    (send dialog :modal-dialog)
    ))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                         ;;
;;         Extensions to Generalized Linear Model          ;;
;;                                             17 Apr 94   ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defmeth glim-proto :DISPLAY-ERROR-RATE ()
  (let ((y (send self :yvar))
        (f (send self :fit-means))  )
    (format t "Error rate = ~6,3f~%"
            (/ (sum (abs (- y (round f))))
               (length y)))
    ))
                 
(defmeth glim-proto :SAVE-ALIST ()
  ; Association list of method-selectors and names for save dialog.
  (list
   '(residual  . :raw-residuals)
   '(deviance  . :deviances)
   '(fitted    . :fit-means)
   '(stanres   . :studentized-residuals)
   '(StudRes   . :externally-studentized-residuals)
   '(Leverage  . :leverages)
   '(CookD     . :cooks-distances)
   '(weights   . :weights)))



;;;;     BOOTSTRAP
;;;;


(defmeth glim-proto :BOOTSTRAP-FUNCTION (method)
   (if (eq 'random method) 
      #'(lambda (i)
          (let* ((y   (send self :yvar))
                 (y*  (select y i))
                 (all (iseq (array-dimension (send self :slot-value 'x) 1)))
                 (x*  (select (send self :slot-value 'x) i all))
                 (m   (binomialreg-model x* y* 1 :verbose nil :print nil))
                 (c   (send m :coef-estimates))
                 (f*  (round (send m :fit-means)))
                 (f   (round (/ (+ 1 (exp
                                      (- (matmult
                                          (send self :x-matrix) c)))))))  )
            (nconc (list  (/ (sum (abs (- y f))) (length y))
                          (/ (sum (abs (- y* f*))) (length y)))
                   (send m :coef-estimates)  )
            ))
       (progn 
        (message-dialog "Fixed resampling not implemented.")
        nil)))


(defmeth glim-proto :BOOTSTRAP (b method)
  (format t "~a resampling method.~%" method)
  (let* ((n      (length (send self :y)))
         (name   (strcat (if (eq method 'random) "Random" "Fixed")
                              " Regr"))
         (func   (send self :bootstrap-function method))  )
    (when func
          (bootstrap name
                     (list func)
                     `(resample (iseq ,n))
                     (send *active-icon-window* :dataset)
                     b
                 :labels (nconc
                          (list 'Popn_Err 'Samp_Err 'Constant)
                          (mapcar
                           #'(lambda (n)
                               (string-to-symbol n :prefix "Coef-"))
                           (send self :predictor-names))
                          ))
          )))

(add-glim-items)
