#|
   Modified version of Luke Tierney's plotcontrols.lsp
   file supplied with XLispStat.

    8 Mar 98 ... Adapted for new regression 3D experiments.
   14 Mar 96 ... Adapt click response in button array.
   22 Feb 96 ... Fix shear index bug.
   22 Jan 96 ... Add zoom and tilt controls.
   18 Jan 96 ... Begin work on button array.
|#

;;;;

(provide "controls")

;;;;

(defmeth graph-proto :add-control (c) (send self :add-overlay c))
(defmeth graph-proto :delete-control (c) (send self :delete-overlay c))

;;;;

(defproto GRAPH-CONTROL-PROTO
  '(action location title) nil graph-overlay-proto)


(defmeth graph-control-proto :location (&optional (new nil set))
  ; ... 14 mar 96
  (when set
        (let ((g (send self :graph))  )
          (when g (send self :erase))
          (setf (slot-value 'location) new)
          (when g (send self :redraw))))
  (if (numberp (first (slot-value 'location)))      
      ; floating overlay on right margin
      (slot-value 'location)
      (if (slot-value 'graph)
          (list (apply #'+ 10 (select (send (slot-value 'graph) :content-rect) '(0 2)))
                (second (slot-value 'location)))
          (list 0 0))
      ))


(defmeth graph-control-proto :title (&optional (new nil set))
  (when set
        (send self :erase)
        (setf (slot-value 'title) new)
        (send self :redraw))
  (slot-value 'title))

(defmeth graph-control-proto :erase ()
  (let ((graph (send self :graph))
        (loc (send self :location))
        (sz (send self :size)))
    (if graph (apply #'send graph :erase-rect (append loc sz)))))

(defmeth graph-control-proto :size () 
  (let ((graph (send self :graph))
        (title (send self :title)))
    (if graph
        (list (+ 10 5 (send graph :text-width title)) 20)
        (list 10 10))))

(defmeth graph-control-proto :redraw ()
  (let* ((graph (send self :graph))
         (loc (send self :location))
         (loc-x (first loc))
         (loc-y (second loc))  )
    (send self :erase)
    (send graph :frame-rect loc-x (+ 5 loc-y) 10 10)
    (send graph :draw-text (send self :title) (+ 15 loc-x) (+ 15 loc-y) 0 0)))

(defmeth graph-control-proto :do-click (x y a b)
  (let* ((graph (send self :graph))
         (loc (send self :location))
         (loc-x (first loc))
         (loc-y (+ 5 (second loc))))
    (when (and (< loc-x x (+ loc-x 10)) (< loc-y y (+ loc-y 10)))
          (send graph :paint-rect (+ 1 loc-x) (+ 1 loc-y) 8 8)
          (send self :do-action (list a b))
          (send graph :while-button-down
                #'(lambda (x y) (send self :do-action nil)) nil)
          (send graph :erase-rect (+ 1 loc-x) (+ 1 loc-y) 8 8)
          t)))

(defmeth graph-control-proto :DO-ACTION (x)
  ; ... 24 Jun 96
  (if (slot-value 'action)
      (funcall (slot-value 'action))
      (sysbeep)
      ))

; ------------------------------------------------------
;  Graph check boxes
; ------------------------------------------------------

(defproto GRAPH-CHECKBOX-PROTO
  '(selected) nil graph-control-proto)


(defmeth graph-checkbox-proto :redraw ()
  (let* ((graph (send self :graph))
         (loc (send self :location))
         (loc-x (first loc))
         (loc-y (second loc))  )
    (send self :erase)
    (send graph :frame-rect loc-x (+ 5 loc-y) 10 10)
    (when (slot-value 'selected)
          (send graph :draw-line loc-x (+  5 loc-y) (+ 9 loc-x) (+ 14 loc-y))
          (send graph :draw-line loc-x (+ 14 loc-y) (+ 9 loc-x) (+  5 loc-y))  )
    (send graph :draw-text (send self :title) (+ 15 loc-x) (+ 15 loc-y) 0 0)
    ))
  

(defmeth graph-checkbox-proto :do-click (x y a b)
  (let* ((graph (send self :graph))
         (loc (send self :location))
         (loc-x (first loc))
         (loc-y (+ 5 (second loc))))
    (when (and (< loc-x x (+ loc-x 10)) (< loc-y y (+ loc-y 10)))
          (setf (slot-value 'selected) (not (slot-value 'selected)))
          (if (slot-value 'selected)
              (progn
               (send graph :draw-line loc-x loc-y       (+ 9 loc-x) (+ 9 loc-y))
               (send graph :draw-line loc-x (+ 9 loc-y) (+ 9 loc-x)      loc-y))
              (send graph :erase-rect (+ 1 loc-x) (+ 1 loc-y) 8 8)   )
          (send self :do-action (list a b))  )
    (slot-value 'selected)
    ))


; ---------------------------------------------------------
;  Rockers, renamed as in the book as two button controls
; ---------------------------------------------------------

(defproto TWO-BUTTON-CONTROL-PROTO () () graph-control-proto)

(defmeth two-button-control-proto :size () 
  (let ((graph (send self :graph))
        (title (send self :title)))
    (if graph
        (list (+ 10 5 10 5 (send graph :text-width title)) 20)
        (list 10 10))))

(defmeth two-button-control-proto :redraw ()
  (let* ((graph (send self :graph))
         (loc (send self :location))
         (loc-x (first loc))
         (loc-y (second loc))
         (title (send self :title)))
    (send self :erase)
    (send graph :frame-rect loc-x (+ 5 loc-y) 10 10)
    (send graph :frame-rect (+ 15 loc-x) (+ 5 loc-y) 10 10)
    (send graph :draw-text title (+ 30 loc-x) (+ 15 loc-y) 0 0)))

(defmeth two-button-control-proto :do-click (x y a b)
  (let* ((graph (send self :graph))
         (loc (send self :location))
         (loc-x1 (first loc))
         (loc-x2 (+ 15 loc-x1))
         (loc-y (+ 5 (second loc))))
    (if (< loc-y y (+ loc-y 10))
        (let* ((arg (cond 
                     ((< loc-x1 x (+ loc-x1 10)) '-)
                     ((< loc-x2 x (+ loc-x2 10)) '+)))
               (loc-x (case arg (- loc-x1) (+ loc-x2))))
          (when arg
                (send graph :paint-rect (+ 1 loc-x) (+ 1 loc-y) 8 8)
                (send self :do-action (list a b) arg)
                (send graph :while-button-down
                      #'(lambda (x y) (send self :do-action nil arg)) nil)
                (send graph :erase-rect (+ 1 loc-x) (+ 1 loc-y) 8 8)
                t)))))

(defmeth two-button-control-proto :do-action (x arg) (sysbeep))


#| ;----------------------------------------------------------------------
   A button array presents an array of buttons.  The char slot must
   hold the right number of characters, one for each button, in row
   major order.  Similarly, the title slot holds a list of strings,
   one for each row.

   The action to be taken when a button is clicked is set by the :do-action
   method.  The default is a system beep.

   By default, if no handleClick function is supplied, then only one button
   is highlighted at a time within the array, whose row and column appear 
   in the choice slot.  Each row/col pair for a choice is treated as a DOT pair.
   The (r.c) pair is held in a list.  Choice always returns a list, and by 
   default it just a cons pair.

   Alternatively, by inputting a function to as the clickHandler, you can
   control how the array maintains which buttons are active.  This function
   gets two arguments... the first is the new click row and column as a dot
   pair, then comes the current list (value of the choice slot).  The function
   should return what is to be maintained in the choice slot.

   ;;;;;;;;;;;;;;;;;  test code for button array  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

   ; --- open a spinner and add some room for margins
   (def g (spin-plot (list (uniform-rand 20) (uniform-rand 20) (uniform-rand 20))))
   (send g :margin 110 0 0 20)


   ; --- this version uses the default click hanlder... mutually exclusive.
   (send g :add-control
         (setf ba (send button-array-proto :new
                        '(10 10) '(3 2)
                        '("C" "P" "C" "P" "C" "P")
                        '("One" "Two" "Three"))))
   (send g :redraw-overlays)

   ; --- this version uses a special handler, so one in each row.
   (send g :add-control
         (setf ba (send button-array-proto :new
                        '(10 10) '(3 2)
                        '("C" "P" "C" "P" "C" "P")
                        '("One" "Two" "Three")
                        :clickHandler #'row-button-handler)))
   (send g :redraw-overlays)

   ;--- check extent rect

   (apply #'send g :frame-rect (append (send ba :location) (send ba :size)))

 
|# ;----------------------------------------------------------------------

(defproto BUTTON-ARRAY-PROTO 
  '(dim chars choice clickHandler action) 
  ()
  graph-control-proto)

(defmeth button-array-proto :ISNEW (loc dim chars labels 
                                        &key clickHandler initialChoice action)
  (setf (slot-value 'clickHandler) clickHandler)
  (setf (slot-value 'action) action)
  (if initialChoice 
      (setf (slot-value 'choice) initialChoice))
  (call-next-method :dim dim :chars chars :title labels 
                    :location loc))

(defmeth button-array-proto :DIM () 
  (slot-value 'dim))

(defmeth button-array-proto :SET-BUTTON-STATE (choices)
  ; Sets the button states.
  ; 23 Jul 96
  (setf (slot-value 'choice) choices)
  )
        
(defmeth button-array-proto :CHOICE (&optional (new nil set))
  ; Calls the click handler method when new choice is set.
  ; ... 12 mar 96
  (when set
        ; (format t "*** new choice being set to ~a~%" new)
        (send self :erase)
        (setf (slot-value 'choice) 
              (if (slot-value 'clickHandler) 
                  (funcall (slot-value 'clickHandler) new (slot-value 'choice))
                  new))
        (send self :redraw)  )
  (slot-value 'choice))

(defmeth button-array-proto :SIZE ()
  ; ... 14 mar 96 ... 23 Jul 96
  (let ((graph (send self :graph))
        (title (send self :title))
        (dim   (send self :dim))   )
    (list (if graph
              (+ (* (second dim) (+ 10 10)) 
                 (max (mapcar #'(lambda (s) (send graph :text-width s))
                              (if title title (list " ")))))
              100)
          (* (first dim) 20))
    ))


(defmeth button-array-proto :REDRAW ()
  ; Crude, but does it.
  ; 18 Jan 96 ... 12 mar 96
  (send self :erase)
  (let* ((graph  (send self :graph))
         (loc    (send self :location))
         (loc-x  (+ 5 (first loc))) ; margin
         (loc-y  (second loc))
         (j      -1)
         (ch     (if (listp (rest (send self :choice)))  ; add wrapper
                     (send self :choice) (list (send self :choice))))
         (dim    (send self :dim))
         (chars  (slot-value 'chars))
         (title  (send self :title)))
    (dotimes (row (first dim))
             (let ((y (+ loc-y (* row 20)))  )
               (dotimes (col (second dim))
                        (let ((x (+ loc-x (* col 15)))  )
                          (send graph :frame-rect x (+ 5 y) 10 10)
                          (send graph :draw-text
                                (select chars (incf j)) (+ 5 x) (+ 4 y) 1 1)
                          (when (member (cons row col) ch :test #'equal)
                                ; (format t "~a choice ~a~%" (cons row col) ch)
                                (send graph :draw-mode 'xor)
                                (send graph :paint-rect
                                      (+ 1 x) (+ 6 y) 8 8)
                                (send graph :draw-mode 'normal)
                                )))
               (send graph :draw-text (select title row)
                     (+ loc-x (* (second dim) 15)) (+ 4 y) 0 1)
               ))))


(defmeth button-array-proto :DO-CLICK (x y a b)
  ; 19 Jan 96 ... 14 mar 96
  (let* ((graph (send self :graph))
         (loc   (send self :location))
         (xLoc  (first loc)) 
         (yLoc  (+ 5 (second loc)))  ; top margin
         (size  (send self :size))  )
     (if (and (< xLoc x (+ xLoc (* 15 (second (send self :dim)))))
              (< yLoc y (+ yLoc (second size)))
              (member (mod (floor (/ (- x xloc) 5)) 3) '(1 2))
              (evenp (floor (/ (- y yLoc) 10)))  )
         (let* ((xIn (floor (/ (- x xLoc) 15)))
                (yIn (floor (/ (- y yLoc) 20)))
                (loc-x (+ xLoc 5 (* 15 xin)))
                (loc-y (+ yLoc (* 20 yin)))  )
           (send self :choice (cons yin xin))
           (send self :do-action yin xin (list a b))
           t)
         )))


(defmeth button-array-proto :DO-ACTION (row col mod)
  ; (format t "~a ~a ~a ~%" row col mod)
  (if (slot-value 'action)
      (funcall (slot-value 'action) row col mod)
      (sysbeep)))

;;;;

   (defun ROW-BUTTON-HANDLER (new old)
     ; (format t "*** handler input new ~a and old ~a~%" new old)
     (if old
         (let* ((row  (first new))
                (pair (assoc row old))  )
           (if pair
               (progn (setf (rest pair) (rest new))  ; watch out for this in place op
                      old)
               (push new old)))
         (list new)                           ; make it into a list
         ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Slider
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defproto SLIDER-CONTROL-PROTO 
  '(index sequence display) () graph-control-proto)

(defmeth slider-control-proto :isnew (sequence &key 
                                               (title "Value")
                                               (display sequence)
                                               (location '(10 20))
                                               (index 0)
                                               graph)
  (call-next-method :title title :location location)
  (send self :sequence sequence :display display)
  (send self :index index)
  (if graph (send graph :add-control self)))

(defmeth slider-control-proto :size () 
  (let ((graph (send self :graph))
        (title (send self :title)))
    (list 100 30)))

(defmeth slider-control-proto :redraw ()
  (let* ((graph (send self :graph))
         (loc (send self :location))
         (loc-x (first loc))
         (loc-y (second loc))
         (w (first (send self :size))))
    (when graph
          (send graph :draw-text (send self :title) loc-x (+ loc-y 15) 0 0)
          (send graph :frame-rect loc-x (+ loc-y 20) w 10)
          (send self :draw-indicator))))

(defmeth slider-control-proto :draw-indicator (&optional index)
  (let* ((graph (send self :graph))
         (loc (send self :location))
         (loc-x (first loc))
         (loc-y (second loc))
         (w (first (send self :size)))
         (min (send self :min))
         (max (send self :max))
         (index (if index index (send self :index)))
         (val (floor (* (- w 7) (/ (- index min) (- max min))))))
    (when graph
          (let ((tw (send graph :text-width (send self :title))))
            (send graph :start-buffering)
            (send graph :erase-rect (+ 1 tw loc-x) loc-y (- w tw) 20)
            (send graph :draw-text 
                  (format nil "~a" (elt (send self :display) index))
                  (+ loc-x w) (+ loc-y 15) 2 0)
            (send graph :buffer-to-screen (+ 1 tw loc-x) loc-y (- w tw) 20))
          (send graph :erase-rect (+ 1 loc-x) (+ 21 loc-y) (- w 2) 8)
          (send graph :paint-rect (+ 1 loc-x val) (+ 21 loc-y) 5 8))))

(defmeth slider-control-proto :min () 0)

(defmeth slider-control-proto :max () (- (length (slot-value 'sequence)) 1))

(defmeth slider-control-proto :sequence (&optional (seq nil set) &key 
                                                   (display seq))
  (when set
        (setf (slot-value 'sequence) (coerce seq 'vector))
        (setf (slot-value 'display) (coerce display 'vector)))
  (slot-value 'sequence))

(defmeth slider-control-proto :display () (slot-value 'display))

(defmeth slider-control-proto :index (&optional (new nil set))
  (if set
      (let* ((new (max (send self :min) (min new (send self :max)))))
        (setf (slot-value 'index) new)
        (send self :draw-indicator)
        (send self :do-action (elt (send self :sequence) new))))
  (slot-value 'index))

(defmeth slider-control-proto :do-click (x y a b)
  (let* ((graph (send self :graph))
         (loc (send self :location))
         (loc-x (nth 0 loc))
         (loc-y (nth 1 loc))
         (w (first (send self :size))))
    (when (and (< loc-x x (+ loc-x w)) (< (+ loc-y 20) y (+ loc-y 30)))
          (let ((pos (+ (floor (* (- w 7) (/ (send self :index) 
                                             (send self :max))))
                        loc-x)))
            (cond
              ((<= pos x (+ pos 5))
               (let ((off (- x pos)))
                 (send graph :while-button-down
                       #'(lambda (x y)
                           (let ((val (max (+ loc-x 1)
                                           (min (- x off) 
                                                (+ loc-x (- w 6))))))
                             (setf pos val)
                             (send self :draw-indicator 
                                   (floor (* (send self :max) 
                                             (/ (- pos loc-x) (- w 7)))))))))
                 (send self :index 
                       (floor (* (send self :max) 
                                 (/ (- pos loc-x) (- w 7))))))
              ((< loc-x x pos)
               (send graph :while-button-down
                     #'(lambda (x y)
                         (let ((pos (+ (floor (* w (/ (send self :index) 
                                                      (send self :max))))
                                       loc-x)))
                           (if (< x pos)
                               (send self :index (- (send self :index) 1)))))
                     nil))
              ((< pos x (+ loc-x w))
               (send graph :while-button-down
                     #'(lambda (x y)
                         (let ((pos (+ (floor (* w (/ (send self :index) 
                                                      (send self :max))))
                                       loc-x)))
                           (if (> x pos)
                               (send self :index (+ (send self :index) 1)))))
                     nil))))
          t)))

;;;;
;;;; Rotation example
;;;;

;;; Rotation around axes

(defproto VIEWPOINT-CONTROL-PROTO '(vp1 vp2) () two-button-control-proto)

(defmeth viewpoint-control-proto :ISNEW (vp1 vp2 title yLoc)
  (setf (slot-value 'vp1) vp1)
  (setf (slot-value 'vp2) vp2)
  (call-next-method  :location (list 10 yLoc) :title title)
  )

(defmeth viewpoint-control-proto :VIEWPOINTS (p1 p2)
  (setf (slot-value 'vp1) p1)
  (setf (slot-value 'vp2) p2))


(defmeth viewpoint-control-proto :do-action (first sign)
  ; (format t ":DO-ACTION, ~a ~a ~%" first sign)
  (send (send self :graph) :viewpoint
        (slot-value (case sign (- 'vp1) (+ 'vp2)))
        ))


; -------------------------------------------------------------
; Spin Rotate Control
; -------------------------------------------------------------

(defproto SPIN-ROTATE-CONTROL-PROTO '(a1 a2) () two-button-control-proto)

(defmeth spin-rotate-control-proto :ISNEW (a1 a2 title yLoc)
  (setf (slot-value 'a1) a1)
  (setf (slot-value 'a2) a2)
  (call-next-method  :location (list 10 yLoc) :title title)
  )

(defmeth spin-rotate-control-proto :AXES (a1 a2)
  (setf (slot-value 'a1) a1)
  (setf (slot-value 'a2) a2)
  )

(defmeth spin-rotate-control-proto :do-action (first sign)
  (let ((graph (send self :graph)))
    (if first
        (let* ((trans (send graph :transformation))
               (angle (send graph :angle)))
          (send graph :idle-on (car first))
          (send graph :slot-value 'rotation-type
                (if trans
                    (make-rotation 
                     (matmult trans (slot-value 'a1))
                     (matmult trans (slot-value 'a2))
                     (case sign (+ angle) (- (- angle))))
                    'roll))))
    (send graph :rotate)))

; -------------------------------------------------------------
; Plot Rocking Control
; -------------------------------------------------------------

(defproto SPIN-ROCK-CONTROL-PROTO () () graph-control-proto)

(defmeth spin-rock-control-proto :isnew ()
  (call-next-method :location '(10 110) :title "Rock Plot"))

(defmeth spin-rock-control-proto :do-action (first) 
  (send (send self :graph) :rock-plot))

(defmeth spin-proto :rock-plot (&optional (k 2))
  (let ((angle (send self :angle)))
    (dotimes (i k) (send self :rotate-2 0 2 angle))
    (dotimes (i (* 2 k)) (send self :rotate-2 0 2 (- angle)))
    (dotimes (i k) (send self :rotate-2 0 2 angle))))


; -------------------------------------------------------------
; Plot Pause Control // Graph Buttons     9 Jul 98
; -------------------------------------------------------------

(defproto GRAPH-BUTTON-PROTO
  '(clickText clickGetText) nil graph-control-proto)

(defmeth graph-button-proto :SIZE () 
  (let ((graph (send self :graph))
        (title (send self :title)))
    (if graph
        (list (+ 10 (send graph :text-width title)) 18)
        (list 10 10))))

(defmeth graph-button-proto :RECT ()
  (append (send self :location) (send self :size))
  )

(defmeth graph-button-proto :REDRAW ()
  (let* ((graph (send self :graph))
         (rect  (send self :rect))
         (loc-x (first rect))
         (loc-y (second rect))  )
    (apply #'send graph :erase-rect rect)
    (apply #'send graph :frame-rect rect)
    (send graph :draw-text (send self :title) (+ 3 loc-x) (+ 12 loc-y) 0 0)))

(defmeth graph-button-proto :DO-CLICK (x y a b)
  ; Pretty specialized to get this kind of info
  (let* ((graph    (send self :graph))
         (rect     (send self :rect))
         (text     (if (and a b)
                       (funcall (slot-value 'clickGetText) graph)
                       (slot-value 'clickText)))
         (oldColor (send graph :draw-color))
         (left     (elt rect 0)) (right (+ left (elt rect 2)))
         (top      (elt rect 1)) (bot   (+ top  (elt rect 3)))  )
    (when (and (< left x right) (< top y bot))
          (send graph :draw-color 'red)
          (apply #'send graph :paint-rect rect)
          (send graph :draw-color 'yellow)
          (apply #'send graph :frame-rect rect)
          (send graph :draw-text text (+ 3 left) (+ 12 top) 0 0)
          (send graph :draw-color oldColor)
          (send self :do-action (list a b))
          (send graph :while-button-down
                #'(lambda (x y)
                    (send self :do-action nil)) nil)
          (send self :redraw)
          t)))

(defmeth graph-button-proto :DO-ACTION (x)
  nil)

; -------------------------------------------------------------
;  Speed Control
; -------------------------------------------------------------

(defproto spin-speed-control-proto () () slider-control-proto)

(defmeth spin-speed-control-proto :isnew ()
  ; Set the index to match the value set by initial angle of plot.
  (call-next-method  (* 4 (iseq 26)) :index 5   
                     :location '(10 70) :title "Speed"))

(defmeth spin-speed-control-proto :do-action (v)
  (let ((graph (send self :graph)))
    (if graph (send graph :angle (/ v 1000)))))


; -------------------------------------------------------------
; Zoom Control
; -------------------------------------------------------------

(defproto ZOOM-CONTROL-PROTO '() () graph-control-proto)

(defmeth zoom-control-proto :isnew ()
  (call-next-method :location '(10 130) :title "Zoom Plot"))


(defmeth zoom-control-proto :do-click (x y a b)
  ; Snaps back after clicking stops by restoring transformation.
  ; 22 Jan 96 
  (let* ((graph (send self :graph))
         (loc (send self :location))
         (loc-x (first loc))
         (loc-y (+ 5 (second loc))))
    (when (and (< loc-x x (+ loc-x 10)) (< loc-y y (+ loc-y 10)))
          (send graph :paint-rect (+ 1 loc-x) (+ 1 loc-y) 8 8)
          (let ((trans (send graph :transformation))  )
            (send self :do-action (list a b))
            (send graph :while-button-down
                  #'(lambda (x y) (send self :do-action nil)) nil)
            (send graph :transformation trans))
          (send graph :erase-rect (+ 1 loc-x) (+ 1 loc-y) 8 8)
          t)))

(defmeth zoom-control-proto :do-action (mods) 
  ; 22 Jan 96 
  (let* ((g (send self :graph))
         (n  (iseq (send g :num-points)))
         (s  (repeat 
              (max (mapcar
                    #'(lambda (d)
                        (max (abs
                              (send g :point-transformed-coordinate d n))))
                    '(0 1)))
              3))  )
    (send g :apply-transformation
          (diagonal (/ 1.03 (+ s (* .95 (- 1 s)))))  )
    ))



; -------------------------------------------------------------
; Shear Control
; -------------------------------------------------------------

(defproto SHEAR-CONTROL-PROTO '() () graph-control-proto)

(defmeth shear-control-proto :isnew ()
  (call-next-method :location '(10 150) :title "Shear Plot"))


(defmeth shear-control-proto :do-click (x y a b)
  ; Snaps back after clicking stops by restoring transformation.
  ; 22 Jan 96 ... 24 Jan 96
  (let* ((graph (send self :graph))
         (loc (send self :location))
         (loc-x (first loc))
         (loc-y (+ 5 (second loc))))
    (when (and (< loc-x x (+ loc-x 10)) (< loc-y y (+ loc-y 10)))
          (let ((oldTrans (send graph :transformation))  )
            (send graph :paint-rect (+ 1 loc-x) (+ 1 loc-y) 8 8)
            (send self :do-action (list a b))
            (send graph :while-button-down
                  #'(lambda (x y) (send self :do-action nil)) nil)
            (send graph :transformation oldTrans)   )
          (send graph :erase-rect (+ 1 loc-x) (+ 1 loc-y) 8 8)
          t)))


(defmeth shear-control-proto :do-action (mods) 
  ; 22 Jan 96 ... 24 Jan 96 ... 22 Feb 96
  (let* ((graph (send self :graph))
         (xin   (first (send graph :slot-value 'vp)))    ; selected var
         (trans (send graph :trans-mat))   )
    (if (send graph :slot-value 'horizontal?)
        (let* ((axis  (matmult trans '(0 0 1)))   )
          (send graph :apply-transformation
                (make-array
                 '(2 2) :initial-contents
                 (list 1 (/ (* -.4 (first axis)) (second axis)) 0 1))))
        (let* ((a     (if (zerop xin) '(1 0 0) '(0 1 0)))
               (axis  (matmult trans a))  )
          (send graph :apply-transformation
                (make-array
                 '(2 2) :initial-contents
                 (list 1 0 (/ (* -.4 (second axis)) (first axis)) 1)))
          ))))



; -------------------------------------------------------------
; Patches to Tierney's code
; -------------------------------------------------------------

                                        
(defmeth spin-proto :isnew (&rest args)
  ; Modify to check for control hiding if I am a regr spin plot.
  ; ... 8 Mar 98 
  (apply #'call-next-method args)
  (if (send self :has-slot 'hiddenControls)
      (unless (member 'motionType (slot-value 'hiddenControls))
              (send self :add-overlay 
                    (send regr-spin-control-overlay-proto :new)))
      (send self :add-overlay (send spin-control-overlay-proto :new))
      ))


;;
;; Spinner control overlay needs to have a spin for regr surface
;;    as well as optional creation of these screen controls.
;;    All changes as noted; rest duplicated for completeness


(defproto regr-spin-control-overlay-proto 
  '(top lefts gap side ascent box-top text-base)
  ()
  graph-overlay-proto)

(defmeth regr-spin-control-overlay-proto :isnew ()
  ; Add extra left for spin button
  ; ... 8 Mar 98
  (setf (slot-value 'gap) 5)
  (setf (slot-value 'side) 10)
  (setf (slot-value 'ascent) (send graph-proto :text-ascent))
  (let ((w1 (send graph-proto :text-width "Pitch"))
        (w2 (send graph-proto :text-width "Roll"))
        (w3 (send graph-proto :text-width "Yaw"))
        (gap (slot-value 'gap))
        (side (slot-value 'side)))
    (setf (slot-value 'lefts)
          (list (* 2 gap)
                (+ (*  3 gap) side)
                (+ (*  6 gap) (* 2 side) w1)
                (+ (*  7 gap) (* 3 side) w1)
                (+ (* 11 gap) (* 4 side) w1 w2)
                (+ (* 12 gap) (* 5 side) w1 w2)
                (+ (* 15 gap) (* 6 side) w1 w2 w3)
                (+ (* 16 gap) (* 7 side) w1 w2 w3)))))

(defmeth regr-spin-control-overlay-proto :resize ()
  ; No changes here
  (let* ((graph (send self :graph))
         (height (send graph :canvas-height))
         (bottom-margin (fourth (send graph :margin)))
         (top (+ (- height bottom-margin) 1))
         (gap (slot-value 'gap))
         (side (slot-value 'side))
         (ascent (send graph :text-ascent))
         (text-base (+ top gap (max side ascent)))
         (box-top (- text-base side)))
    (setf (slot-value 'top) top)
    (setf (slot-value 'text-base) text-base)
    (setf (slot-value 'box-top) box-top)))

(defmeth regr-spin-control-overlay-proto :redraw ()
  ; Add spin to name list
  ; 8 Mar 98
  (let ((graph (slot-value 'graph))
        (top (slot-value 'top))
        (lefts (slot-value 'lefts))
        (gap (slot-value 'gap))
        (side (slot-value 'side))
        (text-base (slot-value 'text-base))
        (box-top (slot-value 'box-top)))
    (send graph :draw-line 0 top (send graph :canvas-width) top)
    (mapcar #'(lambda (x) (send graph :frame-rect x box-top side side))
            lefts)
    (mapcar #'(lambda (s x y) (send graph :draw-string s x y))
            '("Pitch" "Roll" "Yaw" "Spin")
            (+ (select lefts '(1 3 5 7)) gap side) 
            (repeat text-base 4))))

(defmeth regr-spin-control-overlay-proto :do-click (x y m1 m2)
  ; add spin to case statement
  ; 8 Mar 98
  (let ((graph (slot-value 'graph))
        (top (slot-value 'top))
        (lefts (slot-value 'lefts))
        (gap (slot-value 'gap))
        (side (slot-value 'side))
        (text-base (slot-value 'text-base))
        (box-top (slot-value 'box-top)))
    (when (< top y)
          (send graph :idle-on nil)
          (if (< box-top y text-base)
              (let ((i (car (which (< lefts x (+ lefts side)))))
                    (angle (abs (send graph :angle))))
                (when i
                      (send graph :angle (if (oddp i) angle (- angle)))
                       (let ((in (floor (/ i 2)))  )
                        (send graph :rotation-type
                              (case in
                                (0  'pitching)
                                (1  'rolling)
                                (2  'yawing)
                                (3   (send graph :spin-matrix))   )))
                      (send graph :while-button-down
                            #'(lambda (x y) (send graph :rotate))
                            nil)
                      (send graph :idle-on m1))))
          t)))

